/* newmail.c - watch for new mail
**
** This is a drastically simplified version of the ancient newmail program
** that was included with the elm mail client.
** 
**
** Copyright  2015 by Jef Poskanzer <jef@mail.acme.com>.
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
**
** For commentary on this license please see http://acme.com/license.html
*/

#include <unistd.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <paths.h>
#include <sysexits.h>
#include <pwd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>

static int is_logged_in( void );
static void announce( char* from, char* subject );

static uid_t uid;

int
main( int argc, char** argv )
    {
    char* argv0;
    int argn;
    unsigned int interval;
    struct passwd* pw;
    char* username;
    struct stat sb;
    off_t prev_size;
    int fd;
    off_t len;
    char* buf;
    ssize_t r;
    char* bufptr;
    char* bol;
    bool looking_for_subject;
    char* from = (char*) 0;
    char* ws;
    char* subject;
    char *usage = "usage:  %s [-i secs]\n";

    argv0 = argv[0];
    argn = 1;
    interval = 60;
    while ( argn < argc && argv[argn][0] == '-' )
	{
	if ( strcmp( argv[argn], "-i" ) == 0 && argn + 1 < argc )
	    {
	    ++argn;
	    interval = atoi( argv[argn] );
	    }
	else
	    {
	    (void) fprintf( stderr, usage, argv[0] );
	    exit( 1 );
	    }
	++argn;
	}
    if ( argn != argc )
	{
	(void) fprintf( stderr, usage, argv[0] );
	exit( 1 );
	}

    uid = getuid();
    pw = getpwuid( uid );
    if ( pw == (struct passwd*) 0 )
	{
	perror( "getpwuid" );
	exit( EX_OSERR );
	}
    username = pw->pw_name;

    if ( chdir( _PATH_MAILDIR ) == -1 )
	{
	perror( "chdir" );
	exit( EX_OSERR );
	}

    /* Background. */
    switch ( fork() )
	{
	case 0:
	/* Child process. */
	break;
	case -1:
	/* Error. */
	perror( "fork" );
	exit( EX_OSERR );
	default:
	/* Parent process. */
	exit( EX_OK );
	}

    if ( stat( username, &sb ) == -1 )
	prev_size = 0;
    else
	prev_size = sb.st_size;

    while ( is_logged_in() )
	{
	sleep( interval );

	if ( stat( username, &sb ) == -1 )
	    prev_size = 0;
	else if ( sb.st_size != prev_size )
	    {
	    if ( sb.st_size < prev_size )
		prev_size = 0;
	    fd = open( username, O_RDONLY );
	    if ( fd != -1 )
		{
		if ( lseek( fd, prev_size, SEEK_SET ) != -1 )
		    {
		    len = sb.st_size - prev_size;
		    buf = malloc( len + 1 );
		    if ( buf != (char*) 0 )
			{
			r = read( fd, buf, len );
			if ( r == len )
			    {
			    buf[len] = '\0';
			    /* Assume the buffer starts at a BOL.
			    ** This isn't necessarily true.
			    */
			    bol = buf;
			    looking_for_subject = false;
			    for ( bufptr = buf; *bufptr; ++bufptr )
				{
				if ( *bufptr == '\n' )
				    {
				    /* Found an EOL. */
				    *bufptr = '\0';
				    if ( strncmp( bol, "From ", 5 ) == 0 )
					{
					from = bol + 5;
					while ( *from == ' ' || *from == '\t' )
					    ++from;
					ws = from + 1;
					while ( *ws != ' ' && *ws != '\t' )
					    ++ws;
					*ws = '\0';
					looking_for_subject = true;
					}
				    else if ( looking_for_subject && strncmp( bol, "Subject:", 8 ) == 0 )
					{
					subject = bol + 8;
					while ( *subject == ' ' || *subject == '\t' )
					    ++subject;
					looking_for_subject = false;
					/* Ok write it out. */
					announce( from, subject );
					}
				    bol = bufptr + 1;
				    }
				}
			    }
			free( buf );
			}
		    }
		close( fd );
		}
	    prev_size = sb.st_size;
	    }
	}

    exit( EX_OK );
    }

static int
is_logged_in( void )
    {
    struct stat sb;

    return fstat( 2, &sb ) == 0 && sb.st_uid == uid;
    }

static void
announce( char* from, char* subject )
    {
    char line[200];
    char* lineptr;

    /* Format the line. */
    snprintf( line, sizeof(line), ">> Mail from %s - %s", from, subject );
    /* Replace any non-printing chars with dots. */
    for ( lineptr = line; *lineptr != '\0'; ++lineptr )
	if ( ! isprint( *lineptr ) )
	    *lineptr = '.';
    /* And out. */
    printf( "\r\n%s\r\n\n", line );
    }
